@@ -1,7 +1,8 @@
 from ssl import SSLContext
-from typing import Dict, List, Optional, Tuple, Union
+from typing import List, Optional, Tuple, Union
 
 from .._backends.auto import AsyncLock, AsyncSocketStream, AutoBackend
+from .._types import URL, Headers, Origin, TimeoutDict
 from .base import (
     AsyncByteStream,
     AsyncHTTPTransport,
@@ -15,13 +16,15 @@ from .http11 import AsyncHTTP11Connection
 class AsyncHTTPConnection(AsyncHTTPTransport):
     def __init__(
         self,
-        origin: Tuple[bytes, bytes, int],
+        origin: Origin,
         http2: bool = False,
         ssl_context: SSLContext = None,
+        socket: AsyncSocketStream = None,
     ):
         self.origin = origin
         self.http2 = http2
         self.ssl_context = SSLContext() if ssl_context is None else ssl_context
+        self.socket = socket
 
         if self.http2:
             self.ssl_context.set_alpn_protocols(["http/1.1", "h2"])
@@ -44,20 +47,17 @@ class AsyncHTTPConnection(AsyncHTTPTransport):
     async def request(
         self,
         method: bytes,
-        url: Tuple[bytes, bytes, int, bytes],
-        headers: List[Tuple[bytes, bytes]] = None,
+        url: URL,
+        headers: Headers = None,
         stream: AsyncByteStream = None,
-        timeout: Dict[str, Optional[float]] = None,
+        timeout: TimeoutDict = None,
     ) -> Tuple[bytes, int, bytes, List[Tuple[bytes, bytes]], AsyncByteStream]:
         assert url[:3] == self.origin
-
         async with self.request_lock:
             if self.state == ConnectionState.PENDING:
-                try:
-                    await self._connect(timeout)
-                except:
-                    self.connect_failed = True
-                    raise
+                if not self.socket:
+                    self.socket = await self._open_socket(timeout)
+                self._create_connection(self.socket)
             elif self.state in (ConnectionState.READY, ConnectionState.IDLE):
                 pass
             elif self.state == ConnectionState.ACTIVE and self.is_http2:
@@ -68,22 +68,30 @@ class AsyncHTTPConnection(AsyncHTTPTransport):
         assert self.connection is not None
         return await self.connection.request(method, url, headers, stream, timeout)
 
-    async def _connect(
-        self, timeout: Dict[str, Optional[float]] = None,
-    ):
+    async def _open_socket(self, timeout: TimeoutDict = None) -> AsyncSocketStream:
         scheme, hostname, port = self.origin
         timeout = {} if timeout is None else timeout
         ssl_context = self.ssl_context if scheme == b"https" else None
-        socket = await self.backend.open_tcp_stream(
-            hostname, port, ssl_context, timeout
-        )
+        try:
+            return await self.backend.open_tcp_stream(
+                hostname, port, ssl_context, timeout
+            )
+        except Exception:
+            self.connect_failed = True
+            raise
+
+    def _create_connection(self, socket: AsyncSocketStream) -> None:
         http_version = socket.get_http_version()
         if http_version == "HTTP/2":
             self.is_http2 = True
-            self.connection = AsyncHTTP2Connection(socket=socket, backend=self.backend)
+            self.connection = AsyncHTTP2Connection(
+                socket=socket, backend=self.backend, ssl_context=self.ssl_context
+            )
         else:
             self.is_http11 = True
-            self.connection = AsyncHTTP11Connection(socket=socket)
+            self.connection = AsyncHTTP11Connection(
+                socket=socket, ssl_context=self.ssl_context
+            )
 
     @property
     def state(self) -> ConnectionState:
@@ -100,8 +108,7 @@ class AsyncHTTPConnection(AsyncHTTPTransport):
         if self.connection is not None:
             self.connection.mark_as_ready()
 
-    async def start_tls(
-        self, hostname: bytes, timeout: Dict[str, Optional[float]] = None
-    ):
+    async def start_tls(self, hostname: bytes, timeout: TimeoutDict = None) -> None:
         if self.connection is not None:
             await self.connection.start_tls(hostname, timeout)
+            self.socket = self.connection.socket
